/*
 * Copyright (c)2008 Hochschul-Informations-System GmbH
 *
 * $Id: OOoConnectionPool.java,v 1.3 2013-01-07 15:59:33 jauer#his.de Exp $
 *
 * $Log: OOoConnectionPool.java,v $
 * Revision 1.3  2013-01-07 15:59:33  jauer#his.de
 * Refactoring OO-Reporting: secondLanguage, debugging, ...
 *
 * Revision 1.2  2012-12-20 10:33:26  wahrendorff#his.de
 * PDF/A1-a zu PDF 1.4 umgestellt #82643, Warnings und unnötige casts beseitigt.
 *
 * Revision 1.1  2012-12-04 09:37:25  wahrendorff#his.de
 * OOoHelper & OOoConnection Klassen in core verschoben, Template Service Funktionen in OOoReportService ausgelagert, betroffene Spring Bean Konfigurationen angepasst, getestet.
 *
 * Revision 1.1  2012-12-03 14:03:57  wahrendorff#his.de
 * Architekturverstöße aufgelöst #64641
 *
 * Revision 1.2  2011-08-11 09:32:24  reifschneider#his.de
 * OOoRuntimeNotFoundException to indicate missing ooo runtime
 *
 * Revision 1.1  2010-09-29 11:51:16  lustig#his.de
 * Test zu Mehrbenutzerbetrieb des OOReporting
 *
 *
 * Created on 28.09.2010 by lustig#his.de
 */
package de.his.core.util.openoffice;

import java.io.IOException;
import java.net.ConnectException;

import com.sun.star.comp.helper.BootstrapException;
import com.sun.star.connection.ConnectionSetupException;

/**
 * Verwaltung von Connections zum OpenOffice-Server.
 * 
 * @author lustig#his.de
 * @version $Id: OOoConnectionPool.java,v 1.3 2013-01-07 15:59:33 jauer#his.de Exp $ 
 */
public interface OOoConnectionPool {
    
    /**
     * Is thrown if the OOoRuntime is not found.
     * 
     * Company: HIS
     * @author reifschneider
     * @version $Revision: 1.3 $
     */
    public static class OOoRuntimeNotFoundException extends Exception{

        /** SerialVersionUID */
        private static final long serialVersionUID = 1L;
        
    }

    /**
     * @return Connection to OpenOffice-Server. Please release after use!
     * @throws ConnectException 
     * @throws OOoRuntimeNotFoundException 
     * @throws ConnectionSetupException 
     * @throws BootstrapException 
     * @throws IOException 
     */
    public abstract OOoConnection getConnection() throws ConnectException, OOoRuntimeNotFoundException, ConnectionSetupException, IOException, BootstrapException;

    /**
     * Release connection to OpenOffice-Server
     * @param connection
     */
    public abstract void releaseConnection(OOoConnection connection);

    /**
     * kills the OOoServer
     */
    public void destroy();

}