/*
 * Copyright (c)2008 Hochschul-Informations-System GmbH
 *
 * $Id: OOoConnectionPoolImpl.java,v 1.3 2013-01-07 15:59:33 jauer#his.de Exp $
 *
 * $Log: OOoConnectionPoolImpl.java,v $
 * Revision 1.3  2013-01-07 15:59:33  jauer#his.de
 * Refactoring OO-Reporting: secondLanguage, debugging, ...
 *
 * Revision 1.2  2012-12-20 10:33:26  wahrendorff#his.de
 * PDF/A1-a zu PDF 1.4 umgestellt #82643, Warnings und unnötige casts beseitigt.
 *
 * Revision 1.1  2012-12-04 09:37:25  wahrendorff#his.de
 * OOoHelper & OOoConnection Klassen in core verschoben, Template Service Funktionen in OOoReportService ausgelagert, betroffene Spring Bean Konfigurationen angepasst, getestet.
 *
 * Revision 1.1  2012-12-03 14:03:57  wahrendorff#his.de
 * Architekturverstöße aufgelöst #64641
 *
 * Revision 1.11  2012-08-30 16:02:53  wahrendorff#his.de
 * fixes #76078 rückgängig
 *
 * Revision 1.10  2012-08-30 09:45:29  wahrendorff#his.de
 * fixes #76078
 *
 * Revision 1.9  2012-06-22 07:52:19  wahrendorff#his.de
 * Konfiguration der OpenOffice Verbindung vorbereitet
 *
 * Revision 1.8  2012-06-21 11:47:19  wahrendorff#his.de
 * Startroutine verbessert, hilft #71119 zu vermeiden
 *
 * Revision 1.6  2012-02-24 09:24:29  wahrendorff#his.de
 * Startoptionen kompatibel gemacht. Neue Notation ist nicht zu älteren OpenOffice Versionen kompatibel.
 *
 * Revision 1.5  2012-02-21 15:34:34  wahrendorff#his.de
 * depricated parameterformat aktualisiert
 *
 * Revision 1.4  2011-09-26 14:53:55  wahrendorff#his.de
 * Startparameter -nologo -nodefault -norestore -nocrashreport und -nolockeck dem OpenOffice Server Start hinzugefügt. Spart ein paar MB Arbeitsspeicher
 *
 * Revision 1.3  2011-08-11 09:32:24  reifschneider#his.de
 * OOoRuntimeNotFoundException to indicate missing ooo runtime
 *
 * Revision 1.2  2010-10-15 07:54:47  rademacker#his.de
 * Parameter "-headless" zu den oooOptions hinzugefügt
 *
 * Revision 1.1  2010-09-29 11:51:16  lustig#his.de
 * Test zu Mehrbenutzerbetrieb des OOReporting
 *
 *
 * Created on 28.09.2010 by lustig#his.de
 */
package de.his.core.util.openoffice;

import java.io.IOException;
import java.io.Serializable;
import java.net.ConnectException;
import java.net.MalformedURLException;
import java.util.List;

import ooo.connector.server.OOoServer;
import com.google.common.collect.Lists;

import com.sun.star.comp.helper.BootstrapException;
import com.sun.star.connection.ConnectionSetupException;

/**
 * Bean, die Connections zum OpenOffice-Server verwaltet und ggf. Server startet.
 * 
 * Einfache Strategie zunächst:
 * - wenn ein OO-Server verfügbar ist, verwende diesen
 * - ist keiner verfügbar, so starte einen und schließe ihn wieder beim finalisieren der Bean
 * 
 * TODO: Konfigurierbare Strategie zum Start von Servern (z.B. bis zu N parallele Server)
 * TODO: gepoolete Connections
 * 
 * @author lustig#his.de
 * @version $Id: OOoConnectionPoolImpl.java,v 1.3 2013-01-07 15:59:33 jauer#his.de Exp $ 
 */
public class OOoConnectionPoolImpl implements OOoConnectionPool, Serializable {
    /** serialVersionUID */
    private static final long serialVersionUID = 1L;

   // private static Logger logger = //logger.getLogger(OOoConnectionPoolImpl.class);


    /*
     * Connection Properties
     */

    private String hostname = "localhost";//Conf.getString(Conf.CS.SYS.REPORTING_OO_ENDPOINT, "localhost");

    private Integer port = Integer.getInteger("8100");//Conf.getInteger(Conf.CS.SYS.REPORTING_OO_ENDPOINT_PORT, 8100);

    // count of connection retries before timeout, waits 1s (1000ms) between retries
    private Integer connectRetry = Integer.getInteger("60");//Conf.getInteger(Conf.CS.SYS.REPORTING_OO_ENDPOINT_CONNECT_RETRY, 60);

    /** Is automatically determined if empty */
    private String installPath = "";
    
    //private OOoConnection conn = new OOoConnection();


    /*
     * Connection Ressources 
     */

    /** Server, if actively started from this bean */
    private OOoServer oooServer;


    /**
     * No-arg Constructor
     */
    public OOoConnectionPoolImpl() {
        super();
    }


    @Override
    public OOoConnection getConnection() throws OOoRuntimeNotFoundException, ConnectionSetupException, IOException, BootstrapException  {
        // TODO: Try to fetch connection from a Pool
        OOoConnection conn = new OOoConnection();
        try {
            conn.connect(getConnectionString());
        } catch (ConnectException cex) {
            ////logger.warn(cex);
            try {
                // If there is no running server, try to start one and connect to it            
                tryToStartServer();
                //wait(1000);
                // wait until office is started
                for (int i = 0;; ++i) {
                    try {
                        conn.connect(getConnectionString());
                        break;
                    } catch (ConnectException ex) {
                        // Wait 500 ms, then try to connect again, but do not wait longer than 5 min (= 600 * 500 ms) total:
                        // 5 minutes is too long, make it customizable!
                        if (i == connectRetry.intValue()) {
                            throw ex;
                        }
                        try {
                            Thread.sleep(1000);
                        } catch (InterruptedException e) {
                            //logger.error(e);
                        }
                    }
                }
            } catch (OOoRuntimeNotFoundException ex) {
                throw ex;
            }
        }

        return conn;
    }

    /**
     * @throws IOException 
     * @throws BootstrapException 
     * 
     */
    private void tryToStartServer() throws OOoRuntimeNotFoundException, IOException, BootstrapException {

        // TODO: IllegalStateException, wenn oooServer-Variable nicht null ?

        if (!hostname.equals("localhost") && !hostname.equals("127.0.0.1")) {
            //throw exception if System tries to start OpenOffice on a remote Server, it is not possible. 
            //Remote Server HAVE to be started in advance
            //throw new de.his.appserver.service.ServiceException("cs.sys.ooo.server.cannotStartRemoteServer");
            //logger.error("OO ERROR: Cannot start remoteserver. Only localhost is possible atm.");
        }

        // Install-Path ermitteln, falls nicht gesetzt
        if (installPath == null || installPath.trim().isEmpty()) {
            installPath = InstallationFinder.getPath();
            if (installPath == null) {
                //logger.error("OO ERROR: Path to OO Executable could not be found.");
                throw new OOoRuntimeNotFoundException();
            }
        }

        // Versuch, Server zu starten
        try {
            // Create OOo server with additional -nofirststartwizard option
            List<String> oooOptions = Lists.newArrayList();//OOoServer.getDefaultOOoOptions();
            if (installPath.contains("libreoffice")) {
                // new options for LibreOffice
                oooOptions.add("--nologo");
                oooOptions.add("--nodefault");
                oooOptions.add("--norestore");
                oooOptions.add("--nocrashreport");
                oooOptions.add("--nolockcheck");
                oooOptions.add("--nofirststartwizard");
                oooOptions.add("--headless");
            } else {
                // compatibility options for OpenOffice
                oooOptions.add("-nologo");
                oooOptions.add("-nodefault");
                oooOptions.add("-norestore");
                oooOptions.add("-nocrashreport");
                oooOptions.add("-nolockcheck");
                oooOptions.add("-nofirststartwizard");
                oooOptions.add("-headless");
            }
            //logger.debug(""+oooOptions);
            oooServer = new OOoServer(installPath, oooOptions);
            oooServer.start(getAcceptOption());
            

        } catch (MalformedURLException ex) {
            ex.printStackTrace();
            throw ex;
        } catch (BootstrapException ex) {
            ex.printStackTrace();
            throw ex;
        } catch (IOException ex) {
            ex.printStackTrace();
            throw ex;
        }
    }


    /**
     * @return AcceptOptions for Server Start
     */
    protected String getAcceptOption() {
        String acceptOption;
        if (installPath.contains("libreoffice")) {
            acceptOption = "--accept=" + getConnectionString() + ";urp;StarOffice.ServiceManager;";
        } else {
            acceptOption = "-accept=" + getConnectionString() + ";urp;StarOffice.ServiceManager;";
        }
        return acceptOption;
    }


    /**
     * @return ConnectionString for Server Connection
     */
    protected String getConnectionString() {
        return "socket,host=" + hostname + ",port=" + port.toString() + ",tcpNoDelay=1";
    }


    @Override
    public void releaseConnection(OOoConnection connection) {
        
        connection.disconnect();
        //destroy();
        
    }


    /**
     * Called upon bean-initialization
     */
    public void init() {
        // do nothing        
    }


    /**
     * Called upon bean-finalization
     */
    @Override
    public synchronized void destroy() {
        // TODO: Wait for any unreleased connections?

        // TODO: Release any unreleased connections

        // If we started the server ourselves, bring it down
        if (oooServer != null) {
            try {
                OOoConnection conn = getConnection();
                conn.getDesktop().terminate();
                conn.disconnect();
//            } catch (ConnectException ex) {
//                ex.printStackTrace();
//
//            } catch (OOoRuntimeNotFoundException e) {
//                e.printStackTrace();
            } catch (Exception e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }

            oooServer.kill();
            oooServer = null;
        }

        // Release any other Ressources connected to the Server     
    }



    /*
     * Bean Access 
     */


    /**
     * @return the hostname
     */
    public String getHostname() {
        return hostname;
    }


    /**
     * @param hostname the hostname to set
     */
    public void setHostname(String hostname) {
        this.hostname = hostname;
    }


    /**
     * @return the port
     */
    public Integer getPort() {
        return port;
    }


    /**
     * @param port the port to set
     */
    public void setPort(Integer port) {
        this.port = port;
    }


    /**
     * @return the execPath
     */
    public String getExecPath() {
        return installPath;
    }


    /**
     * @param execPath the execPath to set
     */
    public void setExecPath(String execPath) {
        this.installPath = execPath;
    }

}
