/*
 * Copyright (c)2008-2009 Hochschul-Informations-System GmbH
 *
 * $Id: RegQuery.java,v 1.2 2013-07-30 10:41:49 wahrendorff#his.de Exp $
 * $Log: RegQuery.java,v $
 * Revision 1.2  2013-07-30 10:41:49  wahrendorff#his.de
 * Installationspfad für OpenOffice Version ab 4.0 unter Windows finden. fixes #94709
 *
 * Revision 1.1  2012-12-04 09:37:26  wahrendorff#his.de
 * OOoHelper & OOoConnection Klassen in core verschoben, Template Service Funktionen in OOoReportService ausgelagert, betroffene Spring Bean Konfigurationen angepasst, getestet.
 *
 * Revision 1.1  2012-12-03 14:03:57  wahrendorff#his.de
 * Architekturverstöße aufgelöst #64641
 *
 * Revision 1.5  2012-02-29 10:23:43  wahrendorff#his.de
 * findet den Installationspfad zu LibreOffice nun auch im Subkey wow6432node (für Windows Server 64bit)
 *
 * Revision 1.4  2012-02-28 13:31:53  wahrendorff#his.de
 * Findet jetzt auch LibreOffice Installationen auf Windows Kisten.
 *
 * Revision 1.3  2010-10-21 18:02:49  lustig#his.de
 * UNO wird nun auch auf 64bit Windows-Systemen gefunden, wenn nur ein 32bit OO installiert ist.
 *
 * 
 */

package de.his.core.util.openoffice;

import java.io.IOException;
import java.io.InputStream;
import java.io.StringWriter;

/**
 * Queries the Windows Registry for Uno Installations
 * 
 * @author wahrendorff#his.de, reifschneider#his.de
 * @version $Id: RegQuery.java,v 1.2 2013-07-30 10:41:49 wahrendorff#his.de Exp $ 
 */
public class RegQuery {

    private static final String REGQUERY_UTIL = "reg query ";

    private static final String REGSTR_TOKEN = "REG_SZ";

    private static String[] UNO_PATHS = {
                "\"HKCU\\Software\\LibreOffice\\UNO\\InstallPath\\",
                "\"HKLM\\Software\\LibreOffice\\UNO\\InstallPath\\",
                "\"HKLM\\SOFTWARE\\Wow6432Node\\LibreOffice\\UNO\\InstallPath\\",
                "\"HKCU\\Software\\OpenOffice.org\\UNO\\InstallPath\\",
                "\"HKLM\\Software\\OpenOffice.org\\UNO\\InstallPath\\",
                "\"HKCU\\Software\\OpenOffice\\UNO\\InstallPath\\",
                "\"HKLM\\Software\\OpenOffice\\UNO\\InstallPath\\",
                "\"HKLM\\SOFTWARE\\Wow6432Node\\OpenOffice\\UNO\\InstallPath\\",
                "\"HKLM\\SOFTWARE\\Wow6432Node\\OpenOffice.org\\UNO\\InstallPath\\"
               
    };


    /**
     * @return InstallPath or null if not found
     */
    public String getUnoInstallPath() {
        for (String regKey : UNO_PATHS) {
            String path = getPathIntern(regKey);
            if (path != null) {
                return path;
            }
        }

        return null;
    }

    static String getPathIntern(String regKey) {

        String path = null;

        try {
            Process process = Runtime.getRuntime().exec(REGQUERY_UTIL + regKey);
            StreamReader reader = new StreamReader(process.getInputStream());

            reader.start();
            process.waitFor();
            reader.join();

            String result = reader.getResult();
            int p = result.indexOf(REGSTR_TOKEN);

            if (p > -1) {
                path = result.substring(p + REGSTR_TOKEN.length()).trim();
                int im = path.indexOf(REGSTR_TOKEN);
                if (im > 0) path = path.substring(im + REGSTR_TOKEN.length()).trim();
            }
        } catch (Exception e) {
            path = null;
        }
        return path;

    }

    static class StreamReader extends Thread {
        private InputStream is;

        private StringWriter sw;

        StreamReader(InputStream is) {
            this.is = is;
            sw = new StringWriter();
        }

        @Override
        public void run() {
            try {
                int c;
                while ((c = is.read()) != -1)
                    sw.write(c);
            } catch (IOException e) {
                ;
            }
        }

        String getResult() {
            return sw.toString();
        }
    }
}
